@@ -1220,33 +1220,33 @@ class Sequence(Generic[T], Iterable[T]):
             destination[k].append(e)
         return destination
 
-    def foreach(self, action: Callable[[T], None]) -> None:
+    def for_each(self, action: Callable[[T], None]) -> None:
         """
          Invokes [action] function on each element of the given Sequence.
 
          Example 1:
         >>> lst = ['a', 'b', 'c']
-        >>> it(lst).foreach(lambda x: print(x))
+        >>> it(lst).for_each(lambda x: print(x))
         a
         b
         c
         """
         self.on_each(action)
     
-    def parallel_foreach(self, action: Callable[[T], None], max_workers: Optional[int]=None) -> None:
+    def parallel_for_each(self, action: Callable[[T], None], max_workers: Optional[int]=None) -> None:
         """
          Invokes [action] function on each element of the given Sequence in parallel.
 
          Example 1:
         >>> lst = ['a', 'b', 'c']
-        >>> it(lst).parallel_foreach(lambda x: print(x))
+        >>> it(lst).parallel_for_each(lambda x: print(x))
         a
         b
         c
 
          Example 2:
         >>> lst = ['a', 'b', 'c']
-        >>> it(lst).parallel_foreach(lambda x: print(x), max_workers=2)
+        >>> it(lst).parallel_for_each(lambda x: print(x), max_workers=2)
         a
         b
         c
@@ -1466,16 +1466,16 @@ class Sequence(Generic[T], Iterable[T]):
         """
         return separator.join(self)
     
-    def progress_bar(self, bar: Callable[[Iterable[T]], Iterable[T]]) -> Sequence[T]:
+    def progress(self, progress_func: Callable[[Sequence[T]], Iterable[T]]) -> Sequence[T]:
         """
          Returns a Sequence that enable a progress bar for the given Sequence.
         
          Example 1:
         >>> from tqdm import tqdm
         >>> from time import sleep
-        >>> it(range(10)).progress_bar(lambda x: tqdm(x, total=len(x))).parallel_map(lambda x: sleep(0.), max_workers=5).to_list() and None
+        >>> it(range(10)).progress(lambda x: tqdm(x, total=x.len)).parallel_map(lambda x: sleep(0.), max_workers=5).to_list() and None
         """
-        return it(bar(self))
+        return ProgressSequence(self, progress_func)
 
     def to_set(self) -> Set[T]:
         """
@@ -1510,13 +1510,17 @@ class Sequence(Generic[T], Iterable[T]):
         """
         return list(self)
     
-    def __len__(self) -> int:
+    @property
+    def len(self) -> int:
         if not isinstance(self._iter, Sequence) and hasattr(self._iter, '__len__'):
             return len(self._iter)
         if self._cache is None:
             for _ in self:
                 pass
         return len(self._cache)
+    
+    def __len__(self) -> int:
+        return self.len
 
     def __repr__(self) -> str:
         return str(self.to_list())
@@ -1706,7 +1710,6 @@ class GroupingSequence(Sequence[Tuple[K, List[T]]]):
         yield from d.items()
 
 
-
 class CombinationSequence(Sequence[Tuple[T, ...]]):
     def __init__(self, iterable: Iterable[T], n: int) -> None:
         super().__init__(iterable)
@@ -1748,6 +1751,17 @@ class ConcatSequence(Sequence[T]):
             yield from i
 
 
+class ProgressSequence(Sequence[T]):
+    def __init__(self, iterable: Sequence[T], progress_func: Callable[[Iterable[T]], Iterable[T]]) -> None:
+        super().__init__(iterable)
+        self._progress_func = progress_func
+    
+    @property
+    def len(self) -> int:
+        return len(self._iter)
+    
+    def __do_iter__(self) -> Iterator[T]:
+        yield from self._progress_func(self._iter)
 
 def throw(exception: Exception) -> None:
     raise exception
